// CENTER - by Peter Petrov


#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <algorithm>


enum {
	MAX_N = 512,
	MAX_M = 512,
	MAX_L = 128,
	MAX_H = 16384
};

enum {
	KV_EMPTY   = 0,
	KV_BLOCKED = 1,
	KV_CLEANED = 2
};

enum {
	DIR_UP    = 'u',
	DIR_DOWN  = 'd',
	DIR_LEFT  = 'l',
	DIR_RIGHT = 'r'
};

enum {
	MODE_GREEDY = 0,
	MODE_NORMAL = 1
};

typedef unsigned char kv_type;
typedef kv_type matr_type[MAX_N][MAX_M];
struct solution_type {
	int size,value;
	char path[MAX_H];
};
struct restorator {
	int size;
	short x[MAX_L],y[MAX_L];
};
struct dir_value {
	char dir;
	int value;
	inline friend bool operator<(const dir_value& a,const dir_value& b) {
		if (a.value>b.value) return true;
		if (a.value==b.value && (
			(a.dir==DIR_RIGHT&&b.dir==DIR_LEFT)||
			(a.dir==DIR_DOWN&&b.dir==DIR_UP))) return true;
		return false;
	}
};


int N,M,L,H,K;
int solution_version,solution_sec,mode,greedyH,bestVal,bestGreedyVal;
matr_type matr;
solution_type sol,best_sol;
bool is_time_out; int sec_time_out;


inline bool time_out() {
	if (is_time_out) return true;
	if (clock()>sec_time_out*CLOCKS_PER_SEC) { is_time_out=true; return true; } 
	return false;
}

template<class T> void swap(T& a,T& b) { T t=a; a=b; b=a; }

void write_solution(bool flush=false) {
	if (!flush) {
		if (sol.value<=best_sol.value) return;
		best_sol.value=sol.value; best_sol.size=sol.size;
		memcpy(best_sol.path,sol.path,best_sol.size);
	}
	const char *fname;
	int sec=clock()/CLOCKS_PER_SEC;
	if (flush || sec>solution_sec) {
		if ((++solution_version)&1) fname="CENTER.1"; else fname="CENTER.2";
		FILE *fout=fopen(fname,"wt");
		if (!fout) { printf("Can't open %s\n",fname); exit(1); }
		best_sol.path[best_sol.size]=0;
		fprintf(fout,"%s\n",best_sol.path);
		fprintf(fout,"%d\n",solution_version);
		//fprintf(fout,"\n%d\n",best_sol.value);
		fclose(fout);
		solution_sec=sec;
	}
}

int save(int x1,int y1,int x2,int y2,restorator& rest,bool try_only=false) {
	int x,y,count=0; if (!try_only) rest.size=0;
	if (x1>x2) swap(x1,x2); if (y1>y2) swap(y1,y2);
	for(x=x1;x<=x2;++x) for(y=y1;y<=y2;++y) 
		switch(matr[x][y]) {
		case KV_EMPTY:
			if (!try_only) {
				rest.x[rest.size]=x; rest.y[rest.size++]=y;
				matr[x][y]=KV_CLEANED; 
			}
			++count;
			break;
		case KV_BLOCKED: if (!try_only) rest.size=0; return -1; break;
		case KV_CLEANED: default: 
			if (mode==MODE_GREEDY) {
				if (!try_only) rest.size=0; return -1;
			}
			break;
		}
	return count;
};

void restore(const restorator& rest) {
	int i; for(i=0;i<rest.size;++i) 
		matr[rest.x[i]][rest.y[i]]=KV_EMPTY;
}

void recurse(int x,int y) {
	if (sol.size>=H) return;
	//int aVal=(mode==MODE_GREEDY)?bestGreedyVal:bestVal;
	//if (best_sol.value>=aVal) return;
	int aH=(mode==MODE_GREEDY)?greedyH:H;
	if (sol.value+(aH-sol.size)*L<=best_sol.value) return;
	if (time_out()) return;
	int dir,nx,ny; restorator rest;
	dir_value dv[4];
	// UP:
	dv[0].dir=DIR_UP; dv[0].value=-1;
	if (y>0) dv[0].value=save(x,y-1,x+L-1,y-1,rest,true);
	// DOWN:
	dv[1].dir=DIR_DOWN; dv[1].value=-1;
	if (y+L<N) dv[1].value=save(x,y+L,x+L-1,y+L,rest,true);
	// LEFT:
	dv[2].dir=DIR_LEFT; dv[2].value=-1;
	if (x>0) dv[2].value=save(x-1,y,x-1,y+L-1,rest,true);
	// RIGHT:
	dv[3].dir=DIR_RIGHT; dv[3].value=-1;
	if (x+L<M) dv[3].value=save(x+L,y,x+L,y+L-1,rest,true);
	// Sort:
	std::sort(dv,dv+4);
	// Go :)
	for(dir=0;dir<4;++dir) {
		if (dv[dir].value==-1) break;
		switch(dv[dir].dir) {
		case DIR_UP:
			nx=x; ny=y-1; save(x,y-1,x+L-1,y-1,rest); break;
		case DIR_DOWN:
			nx=x; ny=y+1; save(x,y+L,x+L-1,y+L,rest); break;
		case DIR_LEFT:
			nx=x-1; ny=y; save(x-1,y,x-1,y+L-1,rest); break;
		case DIR_RIGHT: default:
			nx=x+1; ny=y; save(x+L,y,x+L,y+L-1,rest); break;
		}
		sol.value+=dv[dir].value; sol.path[sol.size++]=dv[dir].dir; write_solution();
		recurse(nx,ny); sol.value-=dv[dir].value; --sol.size;
		restore(rest); if (is_time_out) break;
	}
}


void main() {
	FILE *fin=fopen("CENTER.INP","rt");
	if (!fin) { printf("Can't open CENTER.INP\n"); exit(1); }
	fscanf(fin,"%d%d%d%d%d",&N,&M,&L,&H,&K);
	int i,x,y,area=M*N; 
	for(x=0;x<M;++x) for(y=0;y<N;++y) matr[x][y]=KV_EMPTY;
	for(i=0;i<K;++i) {
		fscanf(fin,"%d%d",&x,&y);
		matr[y-1][x-1]=KV_BLOCKED; --area;
	}
	fclose(fin);
	//
	greedyH=(area-L*L)/L; if (greedyH>H) greedyH=H;
	bestVal=L*L+H*L; bestGreedyVal=L*L+greedyH*L;
	solution_version=0; solution_sec=-1;
	sol.value=0; sol.size=0; 
	best_sol.value=0; best_sol.size=0;
	for(x=0;x<L;++x) for(y=0;y<L;++y) {
		matr[x][y]=KV_CLEANED; ++sol.value;
	}
	write_solution();
	// GREEDY mode:
	//printf("Entering GREEDY mode.\n");
	sec_time_out=4+clock()/CLOCKS_PER_SEC; 
	if (sec_time_out>15) sec_time_out=15; is_time_out=false; 
	mode=MODE_GREEDY; recurse(0,0);
	//printf("Done GREEDY mode. Value=%d, Size=%d\n",best_sol.value,best_sol.size);
	// NORMAL mode:
	//printf("Entering NORMAL mode.\n");
	sec_time_out=29; is_time_out=false; 
	mode=MODE_NORMAL; recurse(0,0);
	//printf("Done NORMAL mode. Value=%d, Size=%d\n",best_sol.value,best_sol.size);
	// End:
	write_solution(true);
}